@@ -28,6 +28,17 @@
 #include "slot.h"
 #include "menu.h"
 #include "draw.h"
+#include "events.h"
+
+typedef struct
+{
+  SCM lambda;
+  int n_args;
+  SCM arg1;
+  SCM arg2;
+  SCM arg3;
+  SCM retval;
+} CallData;
 
 /* Generic game data */
 typedef struct {
@@ -47,342 +58,7 @@ typedef struct {
 
 lambda_data* game_data = NULL;
 
-/* Scheme to C functions... */
-
-void add_slot(SCM slot_data) 
-{
-  hslot_type hslot = malloc(sizeof(slot_type));
-
-  hslot->id = SCM_INUM(SCM_CAR(slot_data));
-  hslot->cards = new_deck(SCM_CADR(slot_data));
-  hslot->x = scm_num2dbl (SCM_CAR(SCM_CADR(SCM_CADDR(slot_data))), NULL);
-  hslot->y = scm_num2dbl (SCM_CADR(SCM_CADR(SCM_CADDR(slot_data))), NULL);
-  
-  hslot->dx = hslot->dy = hslot->compressed_dy = 0;
-  hslot->expansion_depth = 0;
-
-  if (!strcmp("expanded", SCM_CHARS(SCM_CAR(SCM_CADDR(slot_data))))) {
-    hslot->compressed_dy = hslot->dy = y_expanded_offset;
-  }
-  else if (!strcmp("expanded-right", 
-		   SCM_CHARS(SCM_CAR(SCM_CADDR(slot_data))))) {
-    hslot->dx = x_expanded_offset;
-  }
-  else if (!strcmp("partially-expanded", 
-		   SCM_CHARS(SCM_CAR(SCM_CADDR(slot_data))))) {
-    hslot->compressed_dy = hslot->dy = y_expanded_offset;
-    hslot->expansion_depth = SCM_INUM (SCM_CADDR(SCM_CADDR(slot_data)));
-  }
-  else if (!strcmp("partially-expanded-right", 
-		   SCM_CHARS(SCM_CAR(SCM_CADDR(slot_data))))) {
-    hslot->dx = x_expanded_offset;
-    hslot->expansion_depth = SCM_INUM(SCM_CADDR(SCM_CADDR(slot_data)));
-  }
-
-  hslot->length = hslot->exposed = 0;
-  update_slot_length(hslot);
-
-  slot_list = g_list_append(slot_list, hslot);
-}
-
-hcard_type new_card(SCM card_data) 
-{
-  hcard_type temp_card = malloc(sizeof(card_type));
-
-  temp_card->value = SCM_INUM(SCM_CAR(card_data));
-  temp_card->suit = SCM_INUM(SCM_CADR(card_data));
-  temp_card->direction = !(SCM_NFALSEP(SCM_CADDR(card_data)));
-
-  return temp_card;
-}
-
-GList* new_deck(SCM deck_data) 
-{
-  SCM list_el;
-  GList* temp_deck = NULL;
-
-  if (SCM_NFALSEP(deck_data)) {
-    for (list_el = deck_data; list_el != SCM_EOL; list_el = SCM_CDR(list_el))
-      add_card(&temp_deck,new_card(SCM_CAR(list_el)));
-  }
-  
-  return g_list_reverse(temp_deck);
-}
-
-/* C to Scheme functions... */
-SCM make_card(hcard_type card) 
-{
-  return scm_cons(scm_long2num(card->value),
-		 scm_cons(scm_long2num(card->suit),
-			  scm_cons(SCM_BOOL(!card->direction), SCM_EOL)));
-}
-
-/* Scheme functions */
-static SCM scm_gettext(SCM message)
-{
-  static char * input = NULL;
-  char * output;
-
-  /* This is needed because we can't free the string before returning
-   * if it isn't translated. This way we have a permanent one-string
-   * memory leak and nothing more. */
-  if (!input)
-    free (input);
-  
-  input = SCM_STRING_CHARS (message);
-  output = _(input);
-  return scm_makfrom0str (output);
-}
-
-static SCM scm_undo_set_sensitive (SCM in_state)
-{
-  gboolean state;
-
-  state = SCM_NFALSEP (in_state) ? TRUE : FALSE;
-  undo_set_sensitive (state);
-
-  return SCM_EOL;
-}
-
-static SCM scm_redo_set_sensitive (SCM in_state)
-{
-  gboolean state;
-
-  state = SCM_NFALSEP (in_state) ? TRUE : FALSE;
-  redo_set_sensitive (state);
-
-  return SCM_EOL;
-}
-
-static SCM scm_get_feature_word()
-{
-  return SCM_MAKINUM(enabled_features);
-}
-
-static SCM scm_set_feature_word(SCM features)
-{
-  enabled_features = SCM_INUM(features);
-
-  droppable_is_featured = (enabled_features & DROPPABLE_FMASK);
-  score_is_hidden = (enabled_features & SCOREHIDDEN_FMASK);
-
-  return SCM_EOL;
-}
-
-static SCM scm_set_statusbar_message(SCM message)
-{
-  guint context_id = gtk_statusbar_get_context_id (GTK_STATUSBAR (statusbar), "message");
-  gtk_statusbar_pop (GTK_STATUSBAR (statusbar), context_id);
-  gtk_statusbar_push (GTK_STATUSBAR (statusbar), context_id, _(SCM_STRING_CHARS(message)));
-  return SCM_EOL;
-}
-
-static SCM scm_set_surface_layout(SCM surface) 
-{
-  if (surface != SCM_EOL) {
-    SCM list_el;
-  
-    delete_surface();
-
-    for (list_el = surface; list_el != SCM_EOL; list_el = SCM_CDR(list_el))
-      add_slot(SCM_CAR(list_el));
-  }
-  return SCM_EOL;
-}
-
-
-static SCM scm_reset_surface() 
-{
-  delete_surface();
-  return SCM_EOL;
-}
-
-static SCM gg_scm_add_slot(SCM slot) 
-{
-  add_slot(slot);
-  return SCM_EOL;
-}
-
-static SCM scm_set_slot_y_expansion(SCM scm_slot_id, SCM new_exp_val)
-{
-  hslot_type slot = get_slot(scm_num2int(scm_slot_id, SCM_ARG1, NULL));
-  slot->compressed_dy = slot->dy = scm_num2dbl (new_exp_val, NULL);
-  return SCM_EOL;
-}
-
-static SCM scm_set_slot_x_expansion(SCM scm_slot_id, SCM new_exp_val)
-{
-  hslot_type slot = get_slot(scm_num2int(scm_slot_id, SCM_ARG1, NULL));
-  slot->dx = scm_num2dbl (new_exp_val, NULL);
-  return SCM_EOL;
-}
-
-static SCM scm_get_slot(SCM scm_slot_id) 
-{
-  SCM cards = SCM_EOL;
-  hslot_type slot = get_slot(scm_num2int(scm_slot_id, SCM_ARG1, NULL));
-  GList* tempcard;
-  if (slot) {
-    for (tempcard = slot->cards; tempcard; tempcard = tempcard->next)
-      cards = scm_cons(make_card(tempcard->data), cards);
-
-    cards = scm_cons(scm_slot_id, scm_cons(cards, SCM_EOL));
-  }
-  return cards; 
-}
-
-static SCM scm_set_cards(SCM scm_slot_id, SCM new_cards) 
-{
-  hslot_type hslot = get_slot(scm_num2int(scm_slot_id, SCM_ARG1, NULL));
-  GList* tempptr;
-  
-  for (tempptr = hslot->cards; tempptr; tempptr = tempptr->next)
-    free(tempptr->data);
-
-  hslot->cards = new_deck(new_cards);
-  update_slot_length(hslot);
-
-  return SCM_BOOL_T;
-}
-
-static SCM scm_set_lambda(SCM start_game_lambda, 
-                          SCM pressed_lambda, 
-                          SCM released_lambda, 
-                          SCM clicked_lambda, 
-                          SCM dbl_clicked_lambda,
-                          SCM game_over_lambda,
-                          SCM winning_game_lambda,
-                          SCM hint_lambda,
-                          SCM rest) 
-{
-  if (!game_data)
-	 game_data = malloc(sizeof(lambda_data));
-  game_data->start_game_lambda = start_game_lambda;
-  game_data->button_pressed_lambda = pressed_lambda;
-  game_data->button_released_lambda = released_lambda;
-  game_data->button_clicked_lambda = clicked_lambda;
-  game_data->button_double_clicked_lambda = dbl_clicked_lambda;
-  game_data->game_over_lambda = game_over_lambda;
-  game_data->winning_game_lambda = winning_game_lambda;
-  game_data->hint_lambda = hint_lambda;
-  game_data->get_options_lambda = SCM_CAR(rest);
-  game_data->apply_options_lambda = SCM_CADR(rest);
-  game_data->timeout_lambda = SCM_CADDR(rest);
-
-  if (droppable_is_featured)
-    game_data->droppable_lambda = SCM_CADDDR(rest);
-
-  return SCM_EOL;
-}
-
-static SCM scm_myrandom(SCM range) 
-{
-  return scm_long2num(g_random_int_range(0,SCM_INUM(range)));
-}
-
-static SCM scm_click_to_move_p (void)
-{
-  return click_to_move ? SCM_BOOL_T : SCM_BOOL_F;
-}
-
-static SCM scm_get_score(void) 
-{
-  return scm_long2num(score);
-}
-
-static SCM scm_set_score(SCM new) 
-{
-  score = scm_num2int(new, SCM_ARG1, NULL);
-  set_score();
-  return scm_long2num(score);
-}
-
-static SCM scm_add_to_score(SCM new) 
-{
-  score += scm_num2int(new, SCM_ARG1, NULL);
-  set_score();
-  return scm_long2num(score);
-}
-
-static SCM scm_set_timeout (SCM new) 
-{
-  timeout = scm_num2int(new, SCM_ARG1, NULL);
-  return new;
-}
-
-static SCM scm_get_timeout () 
-{
-  return scm_long2num(timeout);
-}
-
-void cscm_init () 
-{
-  /* Enable useful debugging options. */
-  SCM_DEVAL_P = 1;
-  SCM_BACKTRACE_P = 1;
-  SCM_RECORD_POSITIONS_P = 1;
-  SCM_RESET_DEBUG_MODE;
-
-  /* Let the scheme side of things know about our C functions. */
-  scm_c_define_gsubr("set-feature-word!", 1, 0, 0, scm_set_feature_word);
-  scm_c_define_gsubr("get-feature-word", 0, 0, 0, scm_get_feature_word);
-  scm_c_define_gsubr("set-statusbar-message", 1, 0, 0, scm_set_statusbar_message);
-  scm_c_define_gsubr("set-surface-layout", 1, 0, 0, scm_set_surface_layout);
-  scm_c_define_gsubr("reset-surface", 0, 0, 0, scm_reset_surface);
-  scm_c_define_gsubr("add-slot", 1, 0, 0, gg_scm_add_slot);
-  scm_c_define_gsubr("get-slot", 1, 0, 0, scm_get_slot);  
-  scm_c_define_gsubr("set-cards-c!", 2, 0, 0, scm_set_cards);
-  scm_c_define_gsubr("set-slot-y-expansion!", 2, 0, 0, scm_set_slot_y_expansion);
-  scm_c_define_gsubr("set-slot-x-expansion!", 2, 0, 0, scm_set_slot_x_expansion);
-  scm_c_define_gsubr("set-lambda", 8, 0, 1, scm_set_lambda);
-  scm_c_define_gsubr("random", 1, 0, 0, scm_myrandom);
-  scm_c_define_gsubr("click-to-move?", 0, 0, 0, scm_click_to_move_p);
-  scm_c_define_gsubr("get-score", 0, 0, 0, scm_get_score);  
-  scm_c_define_gsubr("set-score!", 1, 0, 0, scm_set_score);
-  scm_c_define_gsubr("get-timeout", 0, 0, 0, scm_get_timeout);  
-  scm_c_define_gsubr("set-timeout!", 1, 0, 0, scm_set_timeout);
-  scm_c_define_gsubr("add-to-score!", 1, 0, 0, scm_add_to_score);
-  scm_c_define_gsubr("_", 1, 0, 0, scm_gettext);
-  scm_c_define_gsubr("undo-set-sensitive", 1, 0, 0, scm_undo_set_sensitive);
-  scm_c_define_gsubr("redo-set-sensitive", 1, 0, 0, scm_redo_set_sensitive);
-  eval_installed_file ("sol.scm");
-}
-
-typedef struct
-{
-  SCM lambda;
-  int n_args;
-  SCM arg1;
-  SCM arg2;
-  SCM arg3;
-  SCM retval;
-} CallData;
-
-
-static SCM
-cscmi_call_lambda (void *user_data)
-{
-  CallData *call_data = user_data;
-  switch (call_data->n_args)
-    {
-    case 0:
-      call_data->retval = scm_call_0 (call_data->lambda);
-      break;
-    case 1:
-      call_data->retval = scm_call_1 (call_data->lambda, call_data->arg1);
-      break;
-    case 2:
-      call_data->retval = scm_call_2 (call_data->lambda, call_data->arg1, call_data->arg2);
-      break;
-    case 3:
-      call_data->retval = scm_call_3 (call_data->lambda, call_data->arg1, call_data->arg2, call_data->arg3);
-      break;
-    default:
-      g_assert_not_reached ();
-    }
-
-  return SCM_EOL;
-}
+gint32 enabled_features;
 
 static void
 cscmi_write_exception_details (int error_fd,
@@ -520,17 +196,376 @@ cscmi_catch_handler (gpointer data,
   cscmi_write_exception_details (error_fd, tag, throw_args);
   close (error_fd);
 
-  exec_str = g_strdup_printf ("bug-buddy --package=gnome-games --package-version=%s --appname=aisleriot --kill=%d --include=%s",
+  exec_str = g_strdup_printf ("bug-buddy --package=gnome-games --package-ver=%s --appname=aisleriot --pid=%d --kill=%d --include=%s",
 			      VERSION,
 			      getpid (),
+			      getpid (),
 			      error_file);
   system (exec_str);
   unlink (error_file);
   exit (1);
 }
 
-SCM
-cscmi_start_game_lambda (void)
+static SCM
+cscmi_call_lambda (void *user_data)
+{
+  CallData *call_data = user_data;
+  switch (call_data->n_args)
+    {
+    case 0:
+      call_data->retval = scm_call_0 (call_data->lambda);
+      break;
+    case 1:
+      call_data->retval = scm_call_1 (call_data->lambda, call_data->arg1);
+      break;
+    case 2:
+      call_data->retval = scm_call_2 (call_data->lambda, call_data->arg1, call_data->arg2);
+      break;
+    case 3:
+      call_data->retval = scm_call_3 (call_data->lambda, call_data->arg1, call_data->arg2, call_data->arg3);
+      break;
+    default:
+      g_assert_not_reached ();
+    }
+
+  return SCM_EOL;
+}
+
+static SCM c2scm_card(hcard_type card) 
+{
+  return scm_cons(scm_long2num(card->value),
+		 scm_cons(scm_long2num(card->suit),
+			  scm_cons(SCM_BOOL(!card->direction), SCM_EOL)));
+}
+
+static hcard_type scm2c_card(SCM card_data) 
+{
+  hcard_type temp_card = malloc(sizeof(card_type));
+
+  temp_card->value = SCM_INUM(SCM_CAR(card_data));
+  temp_card->suit = SCM_INUM(SCM_CADR(card_data));
+  temp_card->direction = !(SCM_NFALSEP(SCM_CADDR(card_data)));
+
+  return temp_card;
+}
+
+static SCM c2scm_deck(GList *c_cards)
+{
+  SCM scm_cards = SCM_EOL;
+
+  while (c_cards) {
+    scm_cards = scm_cons(c2scm_card((hcard_type)c_cards->data), scm_cards);
+    c_cards = c_cards->next;
+  }
+
+  return scm_cards;
+}
+
+static GList* scm2c_deck(SCM deck_data) 
+{
+  SCM list_el;
+  GList* temp_deck = NULL;
+
+  if (SCM_NFALSEP(deck_data)) {
+    for (list_el = deck_data; list_el != SCM_EOL; list_el = SCM_CDR(list_el))
+      temp_deck = g_list_prepend(temp_deck, scm2c_card(SCM_CAR(list_el)));
+  }
+  
+  return temp_deck;
+}
+
+static void cscmi_add_slot(SCM slot_data) 
+{
+  gboolean expanded_down = FALSE;
+  gboolean expanded_right = FALSE;
+  gint expansion_depth = 0;
+
+  if (!strcmp("expanded", SCM_CHARS(SCM_CAR(SCM_CADDR(slot_data))))) {
+    expanded_down = TRUE;
+  }
+  else if (!strcmp("expanded-right", 
+		   SCM_CHARS(SCM_CAR(SCM_CADDR(slot_data))))) {
+    expanded_right = TRUE;
+  }
+  else if (!strcmp("partially-expanded", 
+		   SCM_CHARS(SCM_CAR(SCM_CADDR(slot_data))))) {
+    expanded_down = TRUE;
+    expansion_depth = SCM_INUM (SCM_CADDR(SCM_CADDR(slot_data)));
+  }
+  else if (!strcmp("partially-expanded-right", 
+		   SCM_CHARS(SCM_CAR(SCM_CADDR(slot_data))))) {
+    expanded_right = TRUE;
+    expansion_depth = SCM_INUM(SCM_CADDR(SCM_CADDR(slot_data)));
+  }
+
+  add_slot(/* id = */ SCM_INUM(SCM_CAR(slot_data)),
+	   /* cards = */ scm2c_deck(SCM_CADR(slot_data)),
+	   /* x = */ scm_num2dbl (SCM_CAR(SCM_CADR(SCM_CADDR(slot_data))), NULL),
+	   /* y = */ scm_num2dbl (SCM_CADR(SCM_CADR(SCM_CADDR(slot_data))), NULL),
+	   expanded_down, expanded_right, expansion_depth);
+}
+
+/* Scheme functions */
+static SCM scm_gettext(SCM message)
+{
+  static char * input = NULL;
+  char * output;
+
+  /* This is needed because we can't free the string before returning
+   * if it isn't translated. This way we have a permanent one-string
+   * memory leak and nothing more. */
+  if (!input)
+    free (input);
+  
+  input = SCM_STRING_CHARS (message);
+  output = _(input);
+  return scm_makfrom0str (output);
+}
+
+static SCM scm_undo_set_sensitive (SCM in_state)
+{
+  gboolean state;
+
+  state = SCM_NFALSEP (in_state) ? TRUE : FALSE;
+  undo_set_sensitive (state);
+
+  return SCM_EOL;
+}
+
+static SCM scm_redo_set_sensitive (SCM in_state)
+{
+  gboolean state;
+
+  state = SCM_NFALSEP (in_state) ? TRUE : FALSE;
+  redo_set_sensitive (state);
+
+  return SCM_EOL;
+}
+
+static SCM scm_get_feature_word()
+{
+  return SCM_MAKINUM(enabled_features);
+}
+
+static SCM scm_set_feature_word(SCM features)
+{
+  enabled_features = SCM_INUM(features);
+
+  droppable_is_featured = (enabled_features & DROPPABLE_FMASK);
+  score_is_hidden = (enabled_features & SCOREHIDDEN_FMASK);
+
+  return SCM_EOL;
+}
+
+static SCM scm_set_statusbar_message(SCM message)
+{
+  guint context_id = gtk_statusbar_get_context_id (GTK_STATUSBAR (statusbar), "message");
+  gtk_statusbar_pop (GTK_STATUSBAR (statusbar), context_id);
+  gtk_statusbar_push (GTK_STATUSBAR (statusbar), context_id, _(SCM_STRING_CHARS(message)));
+  return SCM_EOL;
+}
+
+static SCM scm_set_surface_layout(SCM surface) 
+{
+  if (surface != SCM_EOL) {
+    SCM list_el;
+  
+    delete_all_slots();
+
+    for (list_el = surface; list_el != SCM_EOL; list_el = SCM_CDR(list_el))
+      cscmi_add_slot(SCM_CAR(list_el));
+  }
+  return SCM_EOL;
+}
+
+
+static SCM scm_reset_surface() 
+{
+  delete_all_slots();
+  return SCM_EOL;
+}
+
+static SCM gg_scm_add_slot(SCM slot) 
+{
+  cscmi_add_slot(slot);
+  return SCM_EOL;
+}
+
+static SCM scm_set_slot_y_expansion(SCM scm_slot_id, SCM new_exp_val)
+{
+  hslot_type slot = get_slot(scm_num2int(scm_slot_id, SCM_ARG1, NULL));
+  slot->compressed_dy = slot->dy = scm_num2dbl (new_exp_val, NULL);
+  return SCM_EOL;
+}
+
+static SCM scm_set_slot_x_expansion(SCM scm_slot_id, SCM new_exp_val)
+{
+  hslot_type slot = get_slot(scm_num2int(scm_slot_id, SCM_ARG1, NULL));
+  slot->dx = scm_num2dbl (new_exp_val, NULL);
+  return SCM_EOL;
+}
+
+static SCM scm_get_slot(SCM scm_slot_id) 
+{
+  SCM cards = SCM_EOL;
+  hslot_type slot = get_slot(scm_num2int(scm_slot_id, SCM_ARG1, NULL));
+
+  if (slot) 
+    cards = scm_cons(scm_slot_id, scm_cons(c2scm_deck (slot->cards), SCM_EOL));
+
+  return cards;
+}
+
+static SCM scm_set_cards(SCM scm_slot_id, SCM new_cards) 
+{
+  hslot_type hslot = get_slot(scm_num2int(scm_slot_id, SCM_ARG1, NULL));
+  slot_set_cards(scm2c_deck(new_cards), hslot);
+  return SCM_BOOL_T;
+}
+
+static SCM scm_set_lambda(SCM start_game_lambda, 
+                          SCM pressed_lambda, 
+                          SCM released_lambda, 
+                          SCM clicked_lambda, 
+                          SCM dbl_clicked_lambda,
+                          SCM game_over_lambda,
+                          SCM winning_game_lambda,
+                          SCM hint_lambda,
+                          SCM rest) 
+{
+  if (!game_data)
+	 game_data = malloc(sizeof(lambda_data));
+  game_data->start_game_lambda = start_game_lambda;
+  game_data->button_pressed_lambda = pressed_lambda;
+  game_data->button_released_lambda = released_lambda;
+  game_data->button_clicked_lambda = clicked_lambda;
+  game_data->button_double_clicked_lambda = dbl_clicked_lambda;
+  game_data->game_over_lambda = game_over_lambda;
+  game_data->winning_game_lambda = winning_game_lambda;
+  game_data->hint_lambda = hint_lambda;
+  game_data->get_options_lambda = SCM_CAR(rest);
+  game_data->apply_options_lambda = SCM_CADR(rest);
+  game_data->timeout_lambda = SCM_CADDR(rest);
+
+  if (droppable_is_featured)
+    game_data->droppable_lambda = SCM_CADDDR(rest);
+
+  return SCM_EOL;
+}
+
+static SCM scm_myrandom(SCM range) 
+{
+  return scm_long2num(g_random_int_range(0,SCM_INUM(range)));
+}
+
+static SCM scm_click_to_move_p (void)
+{
+  return click_to_move ? SCM_BOOL_T : SCM_BOOL_F;
+}
+
+static SCM scm_get_score(void) 
+{
+  return scm_long2num(get_score());
+}
+
+static SCM scm_set_score(SCM new) 
+{
+  set_score(scm_num2int(new, SCM_ARG1, NULL));
+  return new;
+}
+
+static SCM scm_add_to_score(SCM delta) 
+{
+  guint new_score = get_score() + scm_num2int(delta, SCM_ARG1, NULL);
+  set_score(new_score);
+  return scm_long2num(new_score);
+}
+
+static SCM scm_set_timeout (SCM new) 
+{
+  timeout = scm_num2int(new, SCM_ARG1, NULL);
+  return new;
+}
+
+static SCM scm_get_timeout () 
+{
+  return scm_long2num(timeout);
+}
+
+static gint scm_execute_delayed_function (SCM callback)
+{
+  CallData *call_data;
+
+  call_data = g_new0 (CallData, 1);
+  call_data->lambda = callback;
+  call_data->n_args = 0;
+  scm_internal_stack_catch (SCM_BOOL_T,
+			    cscmi_call_lambda,
+			    call_data,
+			    cscmi_catch_handler,
+			    NULL);
+  g_free (call_data);
+
+  scm_gc_unprotect_object (callback);
+
+  refresh_screen ();
+
+  end_of_game_test ();
+
+  return FALSE;
+}
+
+static SCM scm_delayed_call (SCM callback)
+{
+  /* This is necessary since guile can't pick up on the fact that 
+   * we're storing the callback value uins g_timeout_add and callback
+   * gets prematurely grabage collected. */
+  scm_gc_protect_object (callback);
+
+  g_timeout_add (50, (GSourceFunc) scm_execute_delayed_function, callback);
+  /* FIXME: We may hae the inhibit the event handling
+   * since this is designed to be called from auto-move
+   * code and there could be a race condition. */
+
+  return SCM_BOOL_T;
+}
+
+void cscm_init () 
+{
+  /* Enable useful debugging options. */
+  SCM_DEVAL_P = 1;
+  SCM_BACKTRACE_P = 1;
+  SCM_RECORD_POSITIONS_P = 1;
+  SCM_RESET_DEBUG_MODE;
+
+  /* Let the scheme side of things know about our C functions. */
+  scm_c_define_gsubr("set-feature-word!", 1, 0, 0, scm_set_feature_word);
+  scm_c_define_gsubr("get-feature-word", 0, 0, 0, scm_get_feature_word);
+  scm_c_define_gsubr("set-statusbar-message", 1, 0, 0, scm_set_statusbar_message);
+  scm_c_define_gsubr("set-surface-layout", 1, 0, 0, scm_set_surface_layout);
+  scm_c_define_gsubr("reset-surface", 0, 0, 0, scm_reset_surface);
+  scm_c_define_gsubr("add-slot", 1, 0, 0, gg_scm_add_slot);
+  scm_c_define_gsubr("get-slot", 1, 0, 0, scm_get_slot);  
+  scm_c_define_gsubr("set-cards-c!", 2, 0, 0, scm_set_cards);
+  scm_c_define_gsubr("set-slot-y-expansion!", 2, 0, 0, scm_set_slot_y_expansion);
+  scm_c_define_gsubr("set-slot-x-expansion!", 2, 0, 0, scm_set_slot_x_expansion);
+  scm_c_define_gsubr("set-lambda", 8, 0, 1, scm_set_lambda);
+  scm_c_define_gsubr("random", 1, 0, 0, scm_myrandom);
+  scm_c_define_gsubr("click-to-move?", 0, 0, 0, scm_click_to_move_p);
+  scm_c_define_gsubr("get-score", 0, 0, 0, scm_get_score);  
+  scm_c_define_gsubr("set-score!", 1, 0, 0, scm_set_score);
+  scm_c_define_gsubr("get-timeout", 0, 0, 0, scm_get_timeout);  
+  scm_c_define_gsubr("set-timeout!", 1, 0, 0, scm_set_timeout);
+  scm_c_define_gsubr("delayed-call", 1, 0, 0, scm_delayed_call);
+  scm_c_define_gsubr("add-to-score!", 1, 0, 0, scm_add_to_score);
+  scm_c_define_gsubr("_", 1, 0, 0, scm_gettext);
+  scm_c_define_gsubr("undo-set-sensitive", 1, 0, 0, scm_undo_set_sensitive);
+  scm_c_define_gsubr("redo-set-sensitive", 1, 0, 0, scm_redo_set_sensitive);
+  eval_installed_file ("sol.scm");
+}
+
+void
+cscmi_start_game_lambda (double *width, double *height)
 {
   CallData *call_data;
   SCM retval;
@@ -546,21 +581,22 @@ cscmi_start_game_lambda (void)
   retval = call_data->retval;
   g_free (call_data);
 
-  return retval;
+  *width = scm_num2double(SCM_CAR (retval), 0, NULL);
+  *height = scm_num2double(SCM_CADR (retval), 0, NULL);
 }
 
-SCM
-cscmi_button_pressed_lambda (SCM slot_id,
-			     SCM cards)
+gboolean
+cscmi_drag_valid(int slot_id, GList *cards)
 {
   CallData *call_data;
   SCM retval;
-
+  
   call_data = g_new0 (CallData, 1);
   call_data->lambda = game_data->button_pressed_lambda;
   call_data->n_args = 2;
-  call_data->arg1 = slot_id;
-  call_data->arg2 = cards;
+  call_data->arg1 = scm_long2num(slot_id);
+  call_data->arg2 = c2scm_deck(cards);
+  scm_gc_protect_object (call_data->arg2);
   scm_internal_stack_catch (SCM_BOOL_T,
 			    cscmi_call_lambda,
 			    call_data,
@@ -568,14 +604,13 @@ cscmi_button_pressed_lambda (SCM slot_id,
 			    NULL);
   retval = call_data->retval;
   g_free (call_data);
+  scm_gc_unprotect_object (call_data->arg2);
 
-  return retval;
+  return SCM_NFALSEP(retval);
 }
 
-SCM
-cscmi_button_released_lambda (SCM start_slot,
-			      SCM cards,
-			      SCM end_slot)
+gboolean
+cscmi_drop_cards(int start_slot, GList *cards, int end_slot)
 {
   CallData *call_data;
   SCM retval;
@@ -583,9 +618,10 @@ cscmi_button_released_lambda (SCM start_slot,
   call_data = g_new0 (CallData, 1);
   call_data->lambda = game_data->button_released_lambda;
   call_data->n_args = 3;
-  call_data->arg1 = start_slot;
-  call_data->arg2 = cards;
-  call_data->arg3 = end_slot;
+  call_data->arg1 = scm_long2num(start_slot);
+  call_data->arg2 = c2scm_deck(cards);
+  scm_gc_protect_object (call_data->arg2);
+  call_data->arg3 = scm_long2num(end_slot);
   scm_internal_stack_catch (SCM_BOOL_T,
 			    cscmi_call_lambda,
 			    call_data,
@@ -593,12 +629,13 @@ cscmi_button_released_lambda (SCM start_slot,
 			    NULL);
   retval = call_data->retval;
   g_free (call_data);
+  scm_gc_unprotect_object (call_data->arg2);
 
-  return retval;
+  return SCM_NFALSEP(retval);
 }
 
-SCM
-cscmi_button_clicked_lambda (SCM slot_id)
+gboolean
+cscmi_button_clicked_lambda(int slot_id)
 {
   CallData *call_data;
   SCM retval;
@@ -606,7 +643,7 @@ cscmi_button_clicked_lambda (SCM slot_id)
   call_data = g_new0 (CallData, 1);
   call_data->lambda = game_data->button_clicked_lambda;
   call_data->n_args = 1;
-  call_data->arg1 = slot_id;
+  call_data->arg1 = scm_long2num(slot_id);
   scm_internal_stack_catch (SCM_BOOL_T,
 			    cscmi_call_lambda,
 			    call_data,
@@ -615,11 +652,11 @@ cscmi_button_clicked_lambda (SCM slot_id)
   retval = call_data->retval;
   g_free (call_data);
 
-  return retval;
+  return SCM_NFALSEP(retval);
 }
 
-SCM
-cscmi_button_double_clicked_lambda (SCM slot_id)
+gboolean
+cscmi_button_double_clicked_lambda(int slot_id)
 {
   CallData *call_data;
   SCM retval;
@@ -627,7 +664,7 @@ cscmi_button_double_clicked_lambda (SCM slot_id)
   call_data = g_new0 (CallData, 1);
   call_data->lambda = game_data->button_double_clicked_lambda;
   call_data->n_args = 1;
-  call_data->arg1 = slot_id;
+  call_data->arg1 = scm_long2num(slot_id);
   scm_internal_stack_catch (SCM_BOOL_T,
 			    cscmi_call_lambda,
 			    call_data,
@@ -636,26 +673,25 @@ cscmi_button_double_clicked_lambda (SCM slot_id)
   retval = call_data->retval;
   g_free (call_data);
 
-  return retval;
+  return SCM_NFALSEP(retval);
 }
 
-SCM
-cscmi_droppable_lambda (SCM start_slot,
-                        SCM cards,
-                        SCM end_slot)
+gboolean
+cscmi_drop_valid(int start_slot, GList *cards, int end_slot)
 {
   CallData *call_data;
   SCM retval;
 
-  if (!droppable_is_featured) 
-    return SCM_BOOL_F;
-  
+  if (!droppable_is_featured)
+    return FALSE;
+
   call_data = g_new0 (CallData, 1);
   call_data->lambda = game_data->droppable_lambda;
   call_data->n_args = 3;
-  call_data->arg1 = start_slot;
-  call_data->arg2 = cards;
-  call_data->arg3 = end_slot;
+  call_data->arg1 = scm_long2num(start_slot);
+  call_data->arg2 = c2scm_deck(cards);
+  scm_gc_protect_object (call_data->arg2);
+  call_data->arg3 = scm_long2num(end_slot);
   scm_internal_stack_catch (SCM_BOOL_T,
 			    cscmi_call_lambda,
 			    call_data,
@@ -663,11 +699,13 @@ cscmi_droppable_lambda (SCM start_slot,
 			    NULL);
   retval = call_data->retval;
   g_free (call_data);
+  scm_gc_unprotect_object (call_data->arg2);
 
-  return retval;
+  return SCM_NFALSEP(retval);
 }
 
-SCM
+
+gboolean
 cscmi_game_over_lambda (void)
 {
   CallData *call_data;
@@ -684,10 +722,10 @@ cscmi_game_over_lambda (void)
   retval = call_data->retval;
   g_free (call_data);
 
-  return retval;
+  return SCM_NFALSEP(retval);
 }
 
-SCM
+gboolean
 cscmi_winning_game_lambda (void)
 {
   CallData *call_data;
@@ -704,7 +742,7 @@ cscmi_winning_game_lambda (void)
   retval = call_data->retval;
   g_free (call_data);
 
-  return retval;
+  return SCM_NFALSEP(retval);
 }
 
 SCM
@@ -748,7 +786,7 @@ cscmi_get_options_lambda (void)
 }
 
 gboolean
-has_options (void)
+cscmi_has_options (void)
 {
   SCM options;
 
@@ -796,3 +834,27 @@ cscmi_timeout_lambda (void)
 
   return SCM_NFALSEP (retval);
 }
+
+
+void cscmi_record_move(int slot_id, GList *old_cards)
+{
+  SCM cards = c2scm_deck(old_cards);
+
+  scm_gc_protect_object (cards);
+
+  scm_call_2(scm_c_eval_string("record-move"),
+	     scm_long2num(slot_id), cards);
+
+  scm_gc_unprotect_object (cards);
+}
+
+void cscmi_end_move(void)
+{
+  scm_call_0(scm_c_eval_string ("end-move"));  
+}
+
+void cscmi_discard_move(void)
+{
+  scm_call_0(scm_c_eval_string ("discard-move"));  
+}
+
