@@ -16,6 +16,10 @@
 ; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 ; USA
 
+(define magic-total 13)
+
+(define waste-slot 1)
+
 (define (new-game)
   (initialize-playing-area)  
   (set-ace-low)
@@ -23,8 +27,6 @@
   (shuffle-deck)
 
   (add-normal-slot DECK)
-  (add-normal-slot '())
-  (set! HORIZPOS (- HORIZPOS (/ 2 3)))
   (add-extended-slot '() right)
 
   (add-carriage-return-slot)
@@ -88,8 +90,8 @@
   (add-normal-slot '())
   (add-normal-slot '())
 
-  (deal-cards 0 '( 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23))
-  (deal-cards-face-up 0 '(24 25 26 27 28 29 30))
+  (deal-cards 0 '(2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22))
+  (deal-cards-face-up 0 '(23 24 25 26 27 28 29))
 
   (give-status-message)
 
@@ -110,18 +112,10 @@
        (not (= (get-value (car card-list)) king))))
 
 (define (check-for-flips slot-id)
-  (cond ((= slot-id 30)
-	 (if (empty-slot? 29)
-	     (flip-top-card 23)
+  (cond ((= slot-id 29)
+	 (if (empty-slot? 28)
+	     (flip-top-card 22)
 	     ))
-	((= slot-id 29)
-	 (begin
-	   (if (empty-slot? 30)
-	       (flip-top-card 23)
-	       )
-	   (if (empty-slot? 28)
-	       (flip-top-card 22)
-	       )))
 	((= slot-id 28)
 	 (begin
 	   (if (empty-slot? 29)
@@ -155,21 +149,21 @@
 	       (flip-top-card 18)
 	       )))
 	((= slot-id 24)
-	 (if (empty-slot? 25)
-	     (flip-top-card 18)
-	     ))
+	 (begin
+	   (if (empty-slot? 25)
+	       (flip-top-card 18)
+	       )
+	   (if (empty-slot? 23)
+	       (flip-top-card 17)
+	       )))
 	((= slot-id 23)
-	 (if (empty-slot? 22)
+	 (if (empty-slot? 24)
 	     (flip-top-card 17)
 	     ))
 	((= slot-id 22)
-	 (begin
-	   (if (empty-slot? 23)
-	       (flip-top-card 17)
-	       )
-	   (if (empty-slot? 21)
-	       (flip-top-card 16)
-	       )))
+	 (if (empty-slot? 21)
+	     (flip-top-card 16)
+	     ))
 	((= slot-id 21)
 	 (begin
 	   (if (empty-slot? 22)
@@ -195,21 +189,21 @@
 	       (flip-top-card 13)
 	       )))
 	((= slot-id 18)
-	 (if (empty-slot? 19)
-	     (flip-top-card 13)
-	     ))
+	 (begin
+	   (if (empty-slot? 19)
+	       (flip-top-card 13)
+	       )
+	   (if (empty-slot? 17)
+	       (flip-top-card 12)
+	       )))
 	((= slot-id 17)
-	 (if (empty-slot? 16)
+	 (if (empty-slot? 18)
 	     (flip-top-card 12)
 	     ))
 	((= slot-id 16)
-	 (begin
-	   (if (empty-slot? 17)
-	       (flip-top-card 12)
-	       )
-	   (if (empty-slot? 15)
-	       (flip-top-card 11)
-	       )))
+	 (if (empty-slot? 15)
+	     (flip-top-card 11)
+	     ))
 	((= slot-id 15)
 	 (begin
 	   (if (empty-slot? 16)
@@ -227,21 +221,21 @@
 	       (flip-top-card 9)
 	       )))
 	((= slot-id 13)
-	 (if (empty-slot? 14)
-	     (flip-top-card 9)
-	     ))
+	 (begin
+	   (if (empty-slot? 14)
+	       (flip-top-card 9)
+	       )
+	   (if (empty-slot? 12)
+	       (flip-top-card 8)
+	       )))
 	((= slot-id 12)
-	 (if (empty-slot? 11)
+	 (if (empty-slot? 13)
 	     (flip-top-card 8)
 	     ))
 	((= slot-id 11)
-	 (begin
-	   (if (empty-slot? 12)
-	       (flip-top-card 8)
-	       )
-	   (if (empty-slot? 10)
-	       (flip-top-card 7)
-	       )))
+	 (if (empty-slot? 10)
+	     (flip-top-card 7)
+	     ))
 	((= slot-id 10)
 	 (begin
 	   (if (empty-slot? 11)
@@ -251,41 +245,47 @@
 	       (flip-top-card 6)
 	       )))
 	((= slot-id 9)
-	 (if (empty-slot? 10)
-	     (flip-top-card 6)
-	     ))
+	 (begin
+	   (if (empty-slot? 10)
+	       (flip-top-card 6)
+	       )
+	   (if (empty-slot? 8)
+	       (flip-top-card 5)
+	       )))
 	((= slot-id 8)
-	 (if (empty-slot? 7)
+	 (if (empty-slot? 9)
 	     (flip-top-card 5)
 	     ))
 	((= slot-id 7)
-	 (begin
-	   (if (empty-slot? 8)
-	       (flip-top-card 5)
-	       )
-	   (if (empty-slot? 6)
-	       (flip-top-card 4)
-	       )))
-	((= slot-id 6)
-	 (if (empty-slot? 7)
+	 (if (empty-slot? 6)
 	     (flip-top-card 4)
 	     ))
+	((= slot-id 6)
+	 (begin
+	   (if (empty-slot? 7)
+	       (flip-top-card 4)
+	       )
+	   (if (empty-slot? 5)
+	       (flip-top-card 3)
+	       )))
 	((= slot-id 5)
-	 (if (empty-slot? 4)
+	 (if (empty-slot? 6)
 	     (flip-top-card 3)
 	     ))
 	((= slot-id 4)
-	 (if (empty-slot? 5)
-	     (flip-top-card 3)
+	 (if (empty-slot? 3)
+	     (flip-top-card 2)
+	     ))
+	((= slot-id 3)
+	 (if (empty-slot? 4)
+	     (flip-top-card 2)
 	     ))
 	))
 
 (define (check-double-flips start-slot end-slot)
   (if (= 1 (- (max start-slot end-slot) (min start-slot end-slot)))
       (let ((smaller (min start-slot end-slot)))
-	(cond ((= smaller 29)
-	       (flip-top-card 23))
-	      ((= smaller 28)
+	(cond ((= smaller 28)
 	       (flip-top-card 22))
 	      ((= smaller 27)
 	       (flip-top-card 21))
@@ -295,7 +295,7 @@
 	       (flip-top-card 19))
 	      ((= smaller 24)
 	       (flip-top-card 18))
-	      ((= smaller 22)
+	      ((= smaller 23)
 	       (flip-top-card 17))
 	      ((= smaller 21)
 	       (flip-top-card 16))
@@ -305,7 +305,7 @@
 	       (flip-top-card 14))
 	      ((= smaller 18)
 	       (flip-top-card 13))
-	      ((= smaller 16)
+	      ((= smaller 17)
 	       (flip-top-card 12))
 	      ((= smaller 15)
 	       (flip-top-card 11))
@@ -313,57 +313,41 @@
 	       (flip-top-card 10))
 	      ((= smaller 13)
 	       (flip-top-card 9))
-	      ((= smaller 11)
+	      ((= smaller 12)
 	       (flip-top-card 8))
 	      ((= smaller 10)
 	       (flip-top-card 7))
 	      ((= smaller 9)
 	       (flip-top-card 6))
-	      ((= smaller 7)
+	      ((= smaller 8)
 	       (flip-top-card 5))
 	      ((= smaller 6)
 	       (flip-top-card 4))
-	      ((= smaller 4)
+	      ((= smaller 5)
 	       (flip-top-card 3))
+	      ((= smaller 3)
+	       (flip-top-card 2))
 	      (#t #t)))))
 
 (define (droppable? start-slot card-list end-slot)
   (and (not (empty-slot? end-slot))
-       (not (= start-slot end-slot))
        (is-visible? (get-top-card end-slot))
-       (= 13 (+ (get-value (car card-list))
-		(get-value (get-top-card end-slot))))))
+       (= magic-total (+ (get-value (car card-list))
+			 (get-value (get-top-card end-slot))))))
 
 (define (button-released start-slot card-list end-slot)
-  (and (droppable? start-slot card-list end-slot)
-       (remove-card end-slot)
-       (check-for-flips start-slot)
-       (check-for-flips end-slot)
-       (check-double-flips start-slot end-slot)
-       (if (or (= start-slot 1)
-	       (= end-slot 1))
-	   (if (not (empty-slot? 2))
-	       (begin
-		 (let ((new-contents (get-cards 2)))
-		   (let ((moving-back (car (reverse new-contents))))
-		     (set-cards! 1 (list moving-back)))
-		   (set-cards! 2 (reverse (cdr (reverse new-contents))))))))
-       (add-to-score! 2)))
+  (if (droppable? start-slot card-list end-slot)
+      (begin
+	(remove-card end-slot)
+	(check-for-flips start-slot)
+	(check-for-flips end-slot)
+	(check-double-flips start-slot end-slot)
+	(add-to-score! 2))))
 
 (define (button-clicked slot-id)
   (if (= slot-id 0)
       (if (not (empty-slot? 0))
-	  (begin
-	    (if (not (empty-slot? 1))
-		(begin
-		  (let ((new-contents (get-cards 2)))
-		    (set-cards! 2 (reverse 
-				   (cons (make-visible (make-card 
-							(get-value (get-top-card 1))
-							(get-suit (get-top-card 1))))
-					 (reverse new-contents)))))
-		  (remove-card 1)))
-	    (deal-cards-face-up 0 '(1)))
+	  (deal-cards-face-up 0 (list waste-slot))
 	  #f)
       (and (not (empty-slot? slot-id))
 	   (is-visible? (get-top-card slot-id))
@@ -371,13 +355,6 @@
 	      king)
 	   (remove-card slot-id)
 	   (check-for-flips slot-id)
-	   (if (= slot-id 1)
-	       (if (not (empty-slot? 2))
-		   (begin
-		     (let ((new-contents (get-cards 2)))
-		       (let ((moving-back (car (reverse new-contents))))
-			 (set-cards! 1 (list moving-back)))
-		       (set-cards! 2 (reverse (cdr (reverse new-contents))))))))
 	   (add-to-score! 1))))
 
 (define (button-double-clicked slot-id)
@@ -390,24 +367,35 @@
 
 (define (game-won)
   (and (empty-slot? 3)
-       (empty-slot? 1)
+       (empty-slot? waste-slot)
        (empty-slot? 0)))
 
+; This doesn't check for a King on top, that is a generic move and
+; check-move handles it. This only checks for the one special case
+; of the top two cards matching.
+(define (check-waste-for-move)
+  (if (and (> (length (get-cards waste-slot)) 2)
+	   (= (+ (get-value (get-nth-card waste-slot 1))
+		 (get-value (get-nth-card waste-slot 2))) 
+	      magic-total))
+      (list 0 (_"Match the top two cards of the waste."))
+      #f))
+
 (define (check-move slot1 slot2)
   (if (or (empty-slot? slot1)
 	  (not (is-visible? (get-top-card slot1))))
-      (if (< slot1 29)
+      (if (< slot1 28)
 	  (check-move (+ 1 slot1) (+ 2 slot1))
 	  #f)
       (if (= king (get-value (get-top-card slot1)))
 	  (list 2 (get-name (get-top-card slot1)) (_"itself"))
 	  (if (or (empty-slot? slot2)
 		  (not (is-visible? (get-top-card slot2)))
-		  (not (= 13 (+ (get-value (get-top-card slot1))
-				(get-value (get-top-card slot2))))))
-	      (if (< slot2 30)
+		  (not (= magic-total (+ (get-value (get-top-card slot1))
+					 (get-value (get-top-card slot2))))))
+	      (if (< slot2 29)
 		  (check-move slot1 (+ 1 slot2))
-		  (if (< slot1 29)
+		  (if (< slot1 28)
 		      (check-move (+ 1 slot1) (+ 2 slot1))
 		      #f))
 	      (list 1 
@@ -420,7 +408,8 @@
       #f))
 
 (define (get-hint)
-  (or (check-move 1 2)  
+  (or (check-waste-for-move) 
+      (check-move waste-slot 2)
       (dealable?)))
 
 (define (get-options) 
