@@ -37,6 +37,8 @@ GtkAction *teleport_action;
 GtkAction *random_action;
 GtkAction *wait_action;
 GtkAction *toolbar_toggle_action;
+GtkAction *fullscreen_action;
+GtkAction *leave_fullscreen_action;
 
 /**********************************************************************/
 /* Function Prototypes                                                */
@@ -51,19 +53,25 @@ static void teleport_cb  (GtkAction *action, gpointer data);
 static void randteleport_cb  (GtkAction *action, gpointer data);
 static void wait_cb (GtkAction *action, gpointer data);
 static void show_toolbar_cb (GtkAction *action, gpointer data);
+static void set_fullscreen_actions (gboolean is_fullscreen);
+static void fullscreen_cb (GtkAction *action);
 /**********************************************************************/
 
 const GtkActionEntry action_entry[] = {
   { "GameMenu", NULL, N_("_Game") },
+  { "ViewMenu", NULL, N_("_View") },
   { "MoveMenu", NULL, N_("_Move") },
   { "SettingsMenu", NULL, N_("_Settings") },
   { "HelpMenu", NULL, N_("_Help") },
-  { "NewGame", GAMES_STOCK_NEW_GAME, NULL, NULL, NULL, G_CALLBACK (new_cb) },
+  { "NewGame", GAMES_STOCK_NEW_GAME, NULL, NULL, N_("Start a new game"), G_CALLBACK (new_cb) },
   { "Scores", GAMES_STOCK_SCORES, NULL, NULL, NULL, G_CALLBACK (scores_cb) },
   { "Quit", GTK_STOCK_QUIT, NULL, NULL, NULL, G_CALLBACK (quit_cb) },
-  { "Teleport", GTK_STOCK_JUMP_TO, N_("_Teleport"), NULL, N_("Teleport, safely if possible"), G_CALLBACK(teleport_cb) },
-  { "Random", GTK_STOCK_JUMP_TO, N_("_Random"), NULL, N_("Teleport randomly"), G_CALLBACK(randteleport_cb) },
+  { "Teleport", GAMES_STOCK_TELEPORT, N_("_Teleport"), NULL, N_("Teleport, safely if possible"), G_CALLBACK(teleport_cb) },
+  { "Random", GAMES_STOCK_RTELEPORT, N_("_Random"), NULL, N_("Teleport randomly"), G_CALLBACK(randteleport_cb) },
   { "Wait", GTK_STOCK_STOP, N_("_Wait"), NULL, N_("Wait for the robots"), G_CALLBACK (wait_cb) },
+  { "Fullscreen", GAMES_STOCK_FULLSCREEN, NULL, NULL, NULL, G_CALLBACK (fullscreen_cb) },
+  { "LeaveFullscreen", GAMES_STOCK_LEAVE_FULLSCREEN, NULL, NULL, NULL, G_CALLBACK (fullscreen_cb) },
+
   { "Preferences", GTK_STOCK_PREFERENCES, NULL, NULL, NULL, G_CALLBACK (properties_cb) },
   { "Contents", GAMES_STOCK_CONTENTS, NULL, NULL, NULL, G_CALLBACK (help_cb) }, 
   { "About", GTK_STOCK_ABOUT, NULL, NULL, NULL, G_CALLBACK (about_cb) }
@@ -85,13 +93,17 @@ const char ui_description[] =
 "      <separator/>"
 "      <menuitem action='Quit'/>"
 "    </menu>"
+"    <menu action='ViewMenu'>"
+"      <menuitem action='ShowToolbar'/>"
+"      <menuitem action='Fullscreen'/>"
+"      <menuitem action='LeaveFullscreen'/>"
+"    </menu>"
 "    <menu action='MoveMenu'>"
 "      <menuitem action='Teleport'/>"
 "      <menuitem action='Random'/>"
 "      <menuitem action='Wait'/>"
 "    </menu>"
 "    <menu action='SettingsMenu'>"
-"      <menuitem action='ShowToolbar'/>"
 "      <menuitem action='Preferences'/>"
 "    </menu>"
 "    <menu action='HelpMenu'>"
@@ -108,27 +120,6 @@ const char ui_description[] =
 "  </toolbar>"
 "</ui>";
 
-/* FIXME: We don't actually use this toolbar anymore, but
- * I still need to move each teleport.png into stock. -rah*/
-GnomeUIInfo toolbar[] = {
-  GNOMEUIINFO_ITEM_STOCK(N_("New"), N_("Start a new game"),
-                         new_cb, GTK_STOCK_NEW),
-  GNOMEUIINFO_SEPARATOR,
-
-  /* FIXME: Someday these should be part of a gnome-games icon theme. */
-
-  { GNOME_APP_UI_ITEM, N_("Teleport"), N_("Teleport, safely if possible"), 
-    teleport_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, "teleport.png",
-    0, 0, NULL},
-
-  { GNOME_APP_UI_ITEM, N_("Random"), N_("Teleport randomly"), 
-    randteleport_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, "rteleport.png",
-    0, 0, NULL},
-
-  GNOMEUIINFO_ITEM_STOCK (N_("Wait"), N_("Wait for the robots"), wait_cb, GTK_STOCK_STOP),
-  GNOMEUIINFO_END
-};
-
 /**********************************************************************/
 /* Function Definitions                                               */
 /**********************************************************************/
@@ -156,9 +147,8 @@ new_cb (GtkAction *action, gpointer  data)
  * @data: Callback data
  *
  * Description:
- * Callback for toolbar menu entry. This does gconf work, while show and
- * hide of toolbar is handled by connect_handle_box_to_toolbar_toggle()
- *
+ * Callback for toolbar menu entry. 
+ * 
  * Returns:
  **/
 static void
@@ -254,13 +244,20 @@ about_cb (GtkAction *action, gpointer data)
 {
   const gchar *authors[]= { "Mark Rae <m.rae@inpharmatica.co.uk>", NULL };
 
+  const gchar *documenters[]= { "Patanjali Somayaji", "Mark Rae <m.rae@inpharmatica.co.uk>", NULL };
+
   gtk_show_about_dialog (GTK_WINDOW (app),
 			 "name", _("Robots"),
 			 "version", VERSION,
-			 "copyright", "Copyright \xc2\xa9 1998-2004 Mark Rae",
+			 "copyright", "Copyright \xc2\xa9 1998-2006 Mark Rae",
+			 "license", "GPL 2+",
                          "comments", _("Based on classic BSD Robots."),
 			 "authors", authors,
+			 "documenters", documenters,
 			 "translator_credits", _("translator-credits"),
+			 "logo-icon-name", "gnome-robots",
+			 "website", "http://www.gnome.org/projects/gnome-games/",
+			 "wrap-license", TRUE,
 			 NULL);
 }
 
@@ -279,6 +276,33 @@ static void wait_cb (GtkAction *action, gpointer data)
   game_keypress (KBD_WAIT);
 }
 
+static void 
+set_fullscreen_actions (gboolean is_fullscreen)
+{
+  gtk_action_set_sensitive (leave_fullscreen_action, is_fullscreen);
+  gtk_action_set_visible (leave_fullscreen_action, is_fullscreen);
+
+  gtk_action_set_sensitive (fullscreen_action, !is_fullscreen);
+  gtk_action_set_visible (fullscreen_action, !is_fullscreen);
+}
+
+static void 
+fullscreen_cb (GtkAction *action)
+{
+  if (action == fullscreen_action) {
+    gtk_window_fullscreen (GTK_WINDOW (app));
+  } else {
+    gtk_window_unfullscreen (GTK_WINDOW (app));
+  }
+}
+
+/* Just in case something else takes us to/from fullscreen. */
+void window_state_cb (GtkWidget *widget, GdkEventWindowState *event)
+{
+  if (event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN)
+    set_fullscreen_actions (event->new_window_state 
+                            & GDK_WINDOW_STATE_FULLSCREEN);
+}
 
 /**
  * create_game_menus
@@ -309,6 +333,12 @@ create_game_menus (GtkUIManager *ui_manager)
   random_action = gtk_action_group_get_action (action_group, "Random");
   wait_action = gtk_action_group_get_action (action_group, "Wait");
   toolbar_toggle_action = gtk_action_group_get_action (action_group, "ShowToolbar");
+  fullscreen_action = gtk_action_group_get_action (action_group, "Fullscreen");
+  leave_fullscreen_action = gtk_action_group_get_action (action_group,
+						         "LeaveFullscreen");
+  set_fullscreen_actions (FALSE);
+
+
   return;
 }
 
@@ -361,22 +391,22 @@ void set_move_menu_sensitivity (gboolean state)
 }
 
 static void
-toggle_toolbar_cb (GtkAction *action, GtkWidget *handle_box)
+toggle_toolbar_cb (GtkAction *action, GtkWidget *toolbar)
 {
   gboolean state;
 
   state = gtk_toggle_action_get_active(GTK_TOGGLE_ACTION (action));
   if (state) {
-    gtk_widget_show (handle_box);
+    gtk_widget_show (toolbar);
   } else {
-    gtk_widget_hide (handle_box);
+    gtk_widget_hide (toolbar);
   }
 }
 
 void
-connect_handle_box_to_toolbar_toggle (GtkWidget *handle_box)
+connect_toolbar_toggle (GtkWidget *toolbar)
 {
-  g_signal_connect (toolbar_toggle_action, "activate", G_CALLBACK (toggle_toolbar_cb), handle_box);
+  g_signal_connect (toolbar_toggle_action, "activate", G_CALLBACK (toggle_toolbar_cb), toolbar);
   gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (toolbar_toggle_action),
 				properties_show_toolbar());
 }
