@@ -22,13 +22,15 @@
 
 #include <gtk/gtk.h>
 #include <gnome.h>
+#include <glib/gi18n.h>
 #include <gconf/gconf-client.h>
-#include <libgnomeui/gnome-window-icon.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <games-clock.h>
 #include <games-gconf.h>
 #include <games-frame.h>
 #include <games-stock.h>
+#include <games-scores.h>
+#include <games-scores-dialog.h>
 
 #include "mahjongg.h"
 #include "drawing.h"
@@ -54,6 +56,7 @@ GtkWidget *tiles_label;
 GtkWidget *toolbar;
 
 GtkAction *pause_action;
+GtkAction *resume_action;
 GtkAction *hint_action;
 GtkAction *redo_action;
 GtkAction *undo_action;
@@ -71,7 +74,7 @@ static gint windowwidth, windowheight;
 GList * tileset_list = NULL;
 
 gchar *tileset = NULL;
-static gchar *mapset = NULL;
+static gint mapset = -1;
 static gchar *score_current_mapset = NULL;
 
 static gchar *selected_tileset = NULL;
@@ -97,12 +100,20 @@ GtkWidget *pref_dialog = NULL;
 /* Has the map been changed ? */
 gboolean new_map = TRUE;
 
+GamesScores *highscores;
+
+static const GamesScoresDescription scoredesc = {NULL,
+						 "Easy",
+						 "mahjongg",
+						 GAMES_SCORES_STYLE_TIME_ASCENDING};
+
 enum {
 	GAME_RUNNING = 0,
+	GAME_WAITING,
 	GAME_WON,
 	GAME_LOST,
 	GAME_DEAD
-} game_over;
+} game_over = GAME_WAITING;
 
 static void clear_undo_queue (void);
 void you_won (void);
@@ -173,21 +184,46 @@ window_state_callback (GtkWidget *widget, GdkEventWindowState *event)
 static void 
 update_menu_sensitivities (void)
 {
-	gtk_action_set_sensitive (pause_action, game_over != GAME_WON);
+	gtk_action_set_sensitive (pause_action, 
+				  game_over != GAME_WON &&
+				  game_over != GAME_WAITING);
 	gtk_action_set_sensitive (restart_action, undo_state);
 
 	if (paused) {
 		gtk_action_set_sensitive (hint_action, FALSE);
 		gtk_action_set_sensitive (undo_action, FALSE);
 		gtk_action_set_sensitive (redo_action, FALSE);
+		gtk_action_set_visible (pause_action, FALSE);
+		gtk_action_set_visible (resume_action, TRUE);
 	} else {
 		gtk_action_set_sensitive (hint_action, moves_left > 0);	
 		gtk_action_set_sensitive (undo_action, undo_state);
 		gtk_action_set_sensitive (redo_action, redo_state);
+		gtk_action_set_visible (pause_action, TRUE);
+		gtk_action_set_visible (resume_action, FALSE);
 	}
+	/* This is a workaround to make sure that all toolbar elements are
+	 * shown properly, due to a bug in GtkToolbar in Gtk+. 
+	 * See Bug #332573 for a description of the workaround.
+	 */ 
+	gtk_widget_queue_draw (toolbar);
+	gtk_widget_queue_resize (toolbar);
+	while (gtk_events_pending ())
+		gtk_main_iteration ();
+	gtk_widget_queue_draw (toolbar);
+	gtk_widget_queue_resize (toolbar);
+	while (gtk_events_pending ())
+		gtk_main_iteration ();
 
 }
 
+static void clock_start (void)
+{
+	games_clock_start (GAMES_CLOCK (chrono));
+	game_over = GAME_RUNNING;
+	update_menu_sensitivities ();
+}
+
 /* Undo and redo sensitivity functionality. */
 static void 
 set_undoredo_state (gboolean undo, gboolean redo)
@@ -296,25 +332,43 @@ bg_colour_callback (GtkWidget *widget, gpointer data)
 			"/apps/mahjongg/bgcolour", tmp, NULL);
 }
 
+static gint get_mapset_index (void)
+{
+	gchar *mapset_name;
+	gint newmapset = -1;
+	gint i;
+
+	mapset_name = gconf_client_get_string (conf_client,
+					       "/apps/mahjongg/mapset",
+					       NULL);
+	for (i=0; i<nmaps; i++) 
+		if (g_utf8_collate (mapset_name, maps[i].name) == 0) 
+			newmapset = i;
+
+	if (newmapset == -1) {  /* Oops: name not found. */
+		if (mapset > 0) /* If we have a valid map, use it. */
+			newmapset = mapset;
+		else            /* Otherwise use the default. */
+			newmapset = 0;
+	} 
+
+	g_free (mapset_name);
+
+	return newmapset;
+}
+
 static void
 mapset_changed_cb (GConfClient *client,
 		   guint        cnxn_id,
 		   GConfEntry  *entry,
 		   gpointer     user_data)
+
 {
 	GtkWidget *dialog;
-	char *mapset_tmp;
 	gint response;
 
-	mapset_tmp = gconf_client_get_string (conf_client,
-			"/apps/mahjongg/mapset",
-			NULL);
 
-	/* We check whether the name is valid later. */
-	if (mapset_tmp != NULL) {
-		g_free (mapset);
-		mapset = mapset_tmp;
-	} 
+	mapset = get_mapset_index ();
 
 	new_map = TRUE;
 
@@ -347,16 +401,13 @@ mapset_changed_cb (GConfClient *client,
 static void
 set_map_selection (GtkWidget *widget, void *data)
 {
-	map *map;
+	gint target_mapset;
 
-	map = maps + gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
+	target_mapset = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
 	
-	g_free (mapset);
-	mapset = g_strdup (map->name);
-
 	gconf_client_set_string (conf_client,
 				 "/apps/mahjongg/mapset",
-				 mapset, NULL);
+				 maps[target_mapset].name, NULL);
 }
 
 static void
@@ -416,17 +467,10 @@ message_flash (gchar *message)
 static void 
 update_score_state ()
 {
-        gchar **names = NULL;
-        gfloat *scores = NULL;
-        time_t *scoretimes = NULL;
-	gint top;
+	GList *top;
 
-	top = gnome_score_get_notable (APPNAME, score_current_mapset,
-				       &names, &scores, &scoretimes);
-	gtk_action_set_sensitive (scores_action, top > 0);
-	g_strfreev (names);
-	g_free (scores);
-	g_free (scoretimes);
+	top = games_scores_get (highscores);
+	gtk_action_set_sensitive (scores_action, top != NULL);
 }
 
 static gint
@@ -498,7 +542,7 @@ tile_event (gint tileno, gint button)
 		return;
 
 	if (!games_clock_get_seconds (GAMES_CLOCK(chrono))) 
-		games_clock_start (GAMES_CLOCK(chrono));
+		clock_start ();
 	
 	switch (button) {
 	case 1:
@@ -581,16 +625,13 @@ fill_tile_menu (GtkWidget *menu, gchar *sdir)
 static void
 fill_map_menu (GtkWidget *menu)
 {
-	gint lp, itemno=0 ;
+	gint lp;
 
-	for (lp=0;lp<nmaps;lp++) {
-		gchar *str = g_strdup(_(maps[lp].name)) ;
-
-		gtk_combo_box_append_text (GTK_COMBO_BOX (menu), str) ;
-		if (!g_ascii_strcasecmp (mapset, maps[lp].name))
-			gtk_combo_box_set_active (GTK_COMBO_BOX (menu), itemno); 
-		itemno++ ;
+	for (lp=0; lp<nmaps; lp++) {
+		gtk_combo_box_append_text (GTK_COMBO_BOX (menu), 
+					   Q_(maps[lp].name)) ;
 	}
+	gtk_combo_box_set_active (GTK_COMBO_BOX (menu), mapset);
 }
 
 void
@@ -646,24 +687,36 @@ you_won (void)
 {
         gint pos;
         time_t seconds;
-        gfloat score;
-        GtkWidget *dialog;
+	GamesScoreValue score;
+        static GtkWidget *dialog = NULL;
+	gchar *message;
 	
         game_over = GAME_WON;
 
         seconds = games_clock_get_seconds (GAMES_CLOCK (chrono));
 
-        score = (seconds / 60) * 1.0 + (seconds % 60) / 100.0;
-	pos = gnome_score_log (score, score_current_mapset, FALSE);
+        score.time_double = (seconds / 60) * 1.0 + (seconds % 60) / 100.0;
+
+	pos = games_scores_add_score (highscores, score);
 	update_menu_sensitivities ();
-        if (pos) {
-                dialog = gnome_scores_display (_(APPNAME_LONG), APPNAME,
-					       score_current_mapset, pos);
-		if (dialog != NULL) {
-			gtk_window_set_transient_for (GTK_WINDOW(dialog),
-						      GTK_WINDOW(window));
-			gtk_window_set_modal (GTK_WINDOW(dialog), TRUE);
+        if (pos > 0) {
+		if (dialog) {
+			gtk_window_present (GTK_WINDOW (dialog));
+		} else {
+			dialog = games_scores_dialog_new (highscores, _("Mahjongg Scores"));
+			games_scores_dialog_set_category_description (GAMES_SCORES_DIALOG (dialog),
+							      _("Map:"));
+			message = g_strdup_printf ("<b>%s</b>\n\n%s",
+						   _("Congratulations!"),
+						   _("Your score has made the top ten."));
+			games_scores_dialog_set_message (GAMES_SCORES_DIALOG (dialog), message);
+			g_free (message);
 		}
+		games_scores_dialog_set_hilight (GAMES_SCORES_DIALOG (dialog), pos);
+		/* FIXME: Quit / New Game choice. */
+
+		gtk_dialog_run  (GTK_DIALOG (dialog));
+		gtk_widget_hide (dialog);
         } 
 }
 
@@ -824,7 +877,7 @@ hint_callback (void)
 {
         gint i, j, free=0, type ;
 
-        if (paused || game_over)
+        if (paused || (game_over != GAME_RUNNING && game_over != GAME_WAITING))
                 return;
 
 	/* This prevents the flashing speeding up if the hint button is
@@ -835,6 +888,7 @@ hint_callback (void)
 	/* Snarfed from check free
 	 * Tile Free is now _so_ much quicker, it is more elegant to do a
 	 * British Library search, and safer. */
+	/* Note: British Library should probably read British Museum.  */
 
 	/* Clear any selection */
 	if (selected_tile < MAX_TILES) {
@@ -869,6 +923,7 @@ hint_callback (void)
                 
 	/* 30s penalty */
 	games_clock_add_seconds(GAMES_CLOCK(chrono), 30);
+	clock_start ();
 }
 
 static void
@@ -886,7 +941,10 @@ about_callback (void)
 		_("Maps:"),
 		"Rexford Newbould",
 		"Krzysztof Foltman",
-		"",
+		NULL
+	};
+
+	const gchar *artists [] = {
 		_("Tiles:"),
 		"Jonathan Buzzard",
 		"Jim Evans",
@@ -896,13 +954,24 @@ about_callback (void)
 		NULL
 	};
 
+	const gchar *documenters [] = {
+		"Eric Baudais",
+		NULL
+	};
+
 	gtk_show_about_dialog (GTK_WINDOW (window),
-			       "name", _("GNOME Mahjongg"), 
+			       "name", _("Mahjongg"), 
 			       "version", VERSION,
 			       "comments", _("A matching game played with Mahjongg tiles."),
 			       "copyright", "Copyright \xc2\xa9 1998-2005 Free Software Foundation, Inc.",
+			       "license", "GPL 2+",
 			       "authors", authors,
+			       "artists", artists,
+			       "documenters", documenters,
 			       "translator_credits", _("translator-credits"),
+			       "logo-icon-name", "gnome-mahjongg",
+			       "website", "http://www.gnome.org/projects/gnome-games/",
+			       "wrap-license", TRUE,
 			       NULL);
 }
 
@@ -923,16 +992,15 @@ pause_callback (void)
 	draw_all_tiles ();
         update_menu_sensitivities ();
 	if (paused) {
-		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (pause_action), TRUE);
                 games_clock_stop (GAMES_CLOCK (chrono));
                 message(_("Game paused"));
         }
         else {
-		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (pause_action), FALSE);
-                games_clock_start (GAMES_CLOCK(chrono));
+                clock_start ();
                 message ("");
         }
 	noloops = FALSE;
+
 }
 
 void ensure_pause_off (void)
@@ -942,24 +1010,37 @@ void ensure_pause_off (void)
 }
 
 static void
-scores_callback (GtkWidget *widget, gpointer data)
+scores_callback (GtkAction *action, gpointer data)
 {
-	GtkWidget *dialog;
+	static GtkWidget *dialog = NULL;
 
-	dialog = gnome_scores_display (_(APPNAME_LONG), APPNAME, score_current_mapset, 0);
-	if (dialog != NULL) {
-		gtk_window_set_transient_for (GTK_WINDOW(dialog),
-					      GTK_WINDOW(window));
-		gtk_window_set_modal (GTK_WINDOW(dialog), TRUE);
+	if (dialog) {
+		gtk_window_present (GTK_WINDOW (dialog));
+	} else {
+		dialog = games_scores_dialog_new (highscores, _("Mahjongg Scores"));
+		games_scores_dialog_set_category_description (GAMES_SCORES_DIALOG (dialog),
+							      _("Map:"));
 	}
+
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_hide (dialog);
 }
 
 static void
 init_game (void)
 {
+	gchar *newtitle;
+
+	newtitle = g_strdup_printf (_("%s - %s"), _("Mahjongg"), 
+				    Q_(maps[mapset].name));
+	gtk_window_set_title (GTK_WINDOW (window), newtitle);
+	g_free (newtitle);
+
+	score_current_mapset = maps[mapset].score_name;
+
         gtk_label_set_text (GTK_LABEL (tiles_label), MAX_TILES_STR);
         update_moves_left ();
-        game_over = GAME_RUNNING;
+        game_over = GAME_WAITING;
         sequence_number = 1 ;
         visible_tiles = MAX_TILES;
         selected_tile = MAX_TILES + 1;
@@ -970,7 +1051,7 @@ init_game (void)
 }
 
 static void 
-new_game_cb (GtkWidget *widget, gpointer data)
+new_game_cb (GtkAction *action, gpointer data)
 {
 	stop_hints ();
 	ensure_pause_off ();
@@ -978,13 +1059,13 @@ new_game_cb (GtkWidget *widget, gpointer data)
 }
 
 static void 
-restart_game_cb (GtkWidget *widget, gpointer data)
+restart_game_cb (GtkAction *action, gpointer data)
 {
 	stop_hints ();
 	restart_game ();
 }
 
-static void quit_cb (GtkWidget *widget, GdkEventAny *e, gpointer data)
+static void quit_cb (GObject *object, gpointer data)
 {
 	gtk_main_quit ();
 }
@@ -1005,7 +1086,7 @@ restart_game (void)
 }
 
 static void
-redo_tile_callback (GtkWidget *widget, gpointer data)
+redo_tile_callback (GtkAction *action, gpointer data)
 {
         gint i, change ;
         char *tmpstr;
@@ -1105,29 +1186,8 @@ clear_undo_queue (void)
 static void
 load_map (void)
 {
-	gchar* name = mapset;
-	gint lp ;
-	gboolean found;
-
-	new_map = FALSE;
-		
-	found = FALSE;
-	for (lp=0;lp<nmaps;lp++)
-		if (g_ascii_strcasecmp (maps[lp].name, name) == 0) {
-			found = TRUE;
-			break;
-		}
-
-	if (!found) {
-		lp = 0;
-		g_free (mapset);
-		mapset = g_strdup (maps[0].name);
-		/* We don't set the gconf key to avoid warning messages appearing multiple times.
-		 * Yes, I know this is a bad excuse. */
-	}
-
-	pos = maps[lp].map ;
-
+	new_map = FALSE;		
+	pos = maps[mapset].map ;
 	generate_dependencies ();
 	calculate_view_geometry ();
 	configure_pixmaps (); 
@@ -1136,6 +1196,7 @@ load_map (void)
 static void
 do_game (void)
 {
+	games_scores_set_category (highscores, maps[mapset].score_name);
 	if (new_map)
 		load_map ();
 	generate_game (g_random_int ()); /* puts in the positions of the tiles */
@@ -1158,8 +1219,7 @@ load_preferences (void)
 {
 	gchar *buf;
 	
-	mapset = gconf_client_get_string (conf_client,
-			"/apps/mahjongg/mapset", NULL);
+	mapset = get_mapset_index ();
 
 	buf = gconf_client_get_string (conf_client,
 			"/apps/mahjongg/bgcolour", NULL) ;
@@ -1187,20 +1247,6 @@ load_preferences (void)
 	load_images (selected_tileset);
 }
 
-static void set_score_file (gchar * mapset)
-{
-	int i;
-
-	/* FIXME: This is a bit ugly, but we only save the name of the
-	   map (and it isn't suitable for generating the scorefile
-	   name. It's also a bit close to code freeze to introduce
-	   gratuitous changes so this is it. */
-	for (i=0; i<nmaps; i++) {
-		if (g_utf8_collate (mapset, maps[i].name) == 0)
-			score_current_mapset = maps[i].score_name;
-	}
-}
-
 void
 new_game (void)
 {
@@ -1210,8 +1256,6 @@ new_game (void)
 
 	init_game ();
 
-	set_score_file (mapset);
-
 	update_score_state ();
 }
 
@@ -1268,15 +1312,17 @@ help_cb (GtkAction *action, gpointer data)
         gnome_help_display ("mahjongg.xml", NULL, NULL);
 }
 
-const GtkActionEntry actions[] = {
+static const GtkActionEntry actions[] = {
         { "GameMenu", NULL, N_("_Game") },
         { "SettingsMenu", NULL, N_("_Settings") },
         { "HelpMenu", NULL, N_("_Help") },
-        { "NewGame", GAMES_STOCK_NEW_GAME, NULL, NULL, NULL, G_CALLBACK (new_game_cb) },
-        { "RestartGame", GAMES_STOCK_RESTART_GAME, NULL, NULL, NULL, G_CALLBACK (restart_game_cb) },
-        { "UndoMove", GAMES_STOCK_UNDO_MOVE, NULL, NULL, NULL, G_CALLBACK (undo_tile_callback) },
-        { "RedoMove", GAMES_STOCK_REDO_MOVE, NULL, NULL, NULL, G_CALLBACK (redo_tile_callback) },
-        { "Hint", GAMES_STOCK_HINT, NULL, NULL, NULL, G_CALLBACK (hint_callback) },
+        { "NewGame", GAMES_STOCK_NEW_GAME, NULL, NULL, N_("Start a new game"), G_CALLBACK (new_game_cb) },
+        { "RestartGame", GAMES_STOCK_RESTART_GAME, NULL, NULL, N_("Restart the current game"), G_CALLBACK (restart_game_cb) },
+	{ "PauseGame", GAMES_STOCK_PAUSE_GAME, NULL, NULL, N_("Pause the game"), G_CALLBACK (pause_callback) },
+	{ "ResumeGame", GAMES_STOCK_RESUME_GAME, NULL, NULL, N_("Resume the paused game"), G_CALLBACK (pause_callback) },
+        { "UndoMove", GAMES_STOCK_UNDO_MOVE, NULL, NULL, N_("Undo the last move"), G_CALLBACK (undo_tile_callback) },
+        { "RedoMove", GAMES_STOCK_REDO_MOVE, NULL, NULL, N_("Redo the last move"), G_CALLBACK (redo_tile_callback) },
+        { "Hint", GAMES_STOCK_HINT, NULL, NULL, N_("Show a hint"), G_CALLBACK (hint_callback) },
         { "Scores", GAMES_STOCK_SCORES, NULL, NULL, NULL, G_CALLBACK (scores_callback) },
         { "Quit", GTK_STOCK_QUIT, NULL, NULL, NULL, G_CALLBACK (quit_cb) },
 	{ "Fullscreen", GAMES_STOCK_FULLSCREEN, NULL, NULL, NULL, G_CALLBACK (fullscreen_callback) },
@@ -1286,18 +1332,18 @@ const GtkActionEntry actions[] = {
         { "About", GTK_STOCK_ABOUT, NULL, NULL, NULL, G_CALLBACK (about_callback) }
 };
 
-const GtkToggleActionEntry toggle_actions[] = {
-        { "ShowToolbar", NULL, N_("_Toolbar"), NULL, N_("Show or hide the toolbar"), G_CALLBACK (show_tb_callback) },
-	{ "PauseGame", GAMES_STOCK_PAUSE_GAME, NULL, NULL, NULL, G_CALLBACK (pause_callback) }
+static const GtkToggleActionEntry toggle_actions[] = {
+        { "ShowToolbar", NULL, N_("_Toolbar"), NULL, N_("Show or hide the toolbar"), G_CALLBACK (show_tb_callback) }
 };
 
-const char *ui_description =
+static const char ui_description[] =
 "<ui>"
 "  <menubar name='MainMenu'>"
 "    <menu action='GameMenu'>"
 "      <menuitem action='NewGame'/>"
 "      <menuitem action='RestartGame'/>"
 "      <menuitem action='PauseGame'/>"
+"      <menuitem action='ResumeGame'/>"
 "      <separator/>"
 "      <menuitem action='UndoMove'/>"
 "      <menuitem action='RedoMove'/>"
@@ -1323,6 +1369,7 @@ const char *ui_description =
 "    <toolitem action='NewGame'/>"
 "    <toolitem action='RestartGame'/>"
 "    <toolitem action='PauseGame'/>"
+"    <toolitem action='ResumeGame'/>"
 "    <separator/>"
 "    <toolitem action='UndoMove'/>"
 "    <toolitem action='RedoMove'/>"
@@ -1332,11 +1379,10 @@ const char *ui_description =
 "</ui>";
 
 
-static GtkUIManager*
-create_ui_manager (const gchar *group)
+static void
+create_menus (GtkUIManager *ui_manager)
 {
         GtkActionGroup *action_group;
-	GtkUIManager   *ui_manager;
 
         action_group = gtk_action_group_new ("group");
 
@@ -1344,16 +1390,11 @@ create_ui_manager (const gchar *group)
         gtk_action_group_add_actions (action_group, actions, G_N_ELEMENTS (actions), window);
         gtk_action_group_add_toggle_actions (action_group, toggle_actions, G_N_ELEMENTS (toggle_actions), window);
 
-
-        ui_manager = gtk_ui_manager_new ();
-
-	games_stock_prepare_for_statusbar_tooltips (ui_manager, 
-						    statusbar);
-
         gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
         gtk_ui_manager_add_ui_from_string (ui_manager, ui_description, -1, NULL);
 	restart_action = gtk_action_group_get_action (action_group, "RestartGame");
 	pause_action = gtk_action_group_get_action (action_group, "PauseGame");
+	resume_action = gtk_action_group_get_action (action_group, "ResumeGame");
 	hint_action = gtk_action_group_get_action (action_group, "Hint");
 	undo_action = gtk_action_group_get_action (action_group, "UndoMove");
 	redo_action = gtk_action_group_get_action (action_group, "RedoMove");
@@ -1366,8 +1407,18 @@ create_ui_manager (const gchar *group)
 
         gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (show_toolbar_action),
         			      gconf_client_get_bool (conf_client, "/apps/mahjongg/show_toolbar", NULL));
+}
 
-        return ui_manager;
+static void init_scores (void)
+{
+	int i;
+
+	highscores = games_scores_new (&scoredesc);
+
+	for (i=0; i<nmaps; i++) {
+		games_scores_add_category (highscores, maps[i].score_name, 
+					   maps[i].name);
+	}
 }
 
 int
@@ -1384,23 +1435,26 @@ main (int argc, char *argv [])
 	GtkUIManager *ui_manager;
 	GtkAccelGroup *accel_group;
 
-	gnome_score_init (APPNAME);
+	GnomeProgram *program;
+
+	setgid_io_init ();
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
-	gnome_program_init (APPNAME, VERSION,
-			    LIBGNOMEUI_MODULE,
-			    argc, argv,
-			    GNOME_PARAM_APP_DATADIR, DATADIR,
-			    NULL);
+	program = gnome_program_init (APPNAME, VERSION,
+				      LIBGNOMEUI_MODULE,
+				      argc, argv,
+				      GNOME_PARAM_APP_DATADIR, DATADIR,
+				      NULL);
 
 	games_stock_init ();
 
-	gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-mahjongg.png");
+	gtk_window_set_default_icon_name ("gnome-mahjongg");
 
 	load_maps ();
+	init_scores ();
 
 	conf_client = gconf_client_get_default ();
 	if (!games_gconf_sanity_check_string (conf_client, "/apps/mahjongg/tileset")) {
@@ -1413,7 +1467,6 @@ main (int argc, char *argv [])
         
 	gtk_window_set_default_size (GTK_WINDOW (window), windowwidth,
 				     windowheight);
-	gtk_window_set_title (GTK_WINDOW (window), _("Mahjongg"));
 	g_signal_connect (G_OBJECT (window), "configure_event",
 			  G_CALLBACK (window_configure_cb), NULL);
 	g_signal_connect (G_OBJECT (window), "window_state_event",
@@ -1451,8 +1504,12 @@ main (int argc, char *argv [])
 
 	/* show the status bar items */
 	statusbar = gtk_statusbar_new();
+	ui_manager = gtk_ui_manager_new ();
+
+	games_stock_prepare_for_statusbar_tooltips (ui_manager, statusbar);
 	gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (statusbar), FALSE);
-	ui_manager = create_ui_manager ("MahjonggActions");
+
+	create_menus (ui_manager);
 	accel_group = gtk_ui_manager_get_accel_group (ui_manager);
 	gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
 	box = gtk_ui_manager_get_widget (ui_manager, "/MainMenu");
@@ -1480,7 +1537,6 @@ main (int argc, char *argv [])
 
 	do_game ();
 	init_game ();
-	set_score_file (mapset);
 	update_score_state ();
 	
 	/* Note: we have to have a layout loaded before here so that the
@@ -1494,6 +1550,10 @@ main (int argc, char *argv [])
   	message_flash (_("Remove matching pairs of tiles.")); 
 
 	gtk_main ();
+
+	gnome_accelerators_sync();
+
+	g_object_unref (program);
 	
 	return 0;
 }
